unit frmuMain;

{$DEFINE USE_SKREGEXP}

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes,
  {$IFDEF USE_SKREGEXP}
  SkRegularExpressions,
  {$ELSE}
  RegularExpressions,
  {$ENDIF}
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Dialogs, FMX.Objects, FMX.Grid,
  FMX.Memo, FMX.Edit, FMX.Layouts;

type
{ TStringColumn_Center }
  TStringColumn_Center = class(FMX.Grid.TStringColumn)
  protected
    function CreateCellControl: TStyledControl; override;
  end;

 { TStringColumn_Right }
  TStringColumn_Right = class(FMX.Grid.TStringColumn)
  protected
    function CreateCellControl: TStyledControl; override;
  end;

type
  TfrmMain = class(TForm)
    pnlHeader: TPanel;
    Splitter1: TSplitter;
    chbCompiled: TCheckBox;
    chbIgnoreCase: TCheckBox;
    chbIgnorePatternSpace: TCheckBox;
    chbMultiLine: TCheckBox;
    chbSingleLine: TCheckBox;
    chbExplicitCapture: TCheckBox;
    lblExpression: TLabel;
    sbStatus: TStatusBar;
    sgMatchResult: TStringGrid;
    btnMatch: TButton;
    btnNextMatch: TButton;
    meInputString: TMemo;
    edExpression: TEdit;
    btnEscape: TButton;
    lblSimpleText: TLabel;
    StyleBook1: TStyleBook;
    procedure FormShow(Sender: TObject);
    procedure edExpressionChange(Sender: TObject);
    procedure btnMatchClick(Sender: TObject);
    procedure btnNextMatchClick(Sender: TObject);
    procedure chb_Click(Sender: TObject);
    procedure meInputStringChange(Sender: TObject);
    procedure btnEscapeClick(Sender: TObject);
  private
    { Private 錾 }
    RegExp: TRegEx;
    Match: TMatch;
    procedure ShowResult(aMatch: Boolean);
    procedure ClearResult;
  public
    { Public 錾 }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.FMX}

{ TStringColumn_Center }
function TStringColumn_Center.CreateCellControl: TStyledControl;
begin
  result := inherited;
  TEdit(result).TextAlign := TTextAlign.taCenter;
end;

{ TStringColumn_Right }
function TStringColumn_Right.CreateCellControl: TStyledControl;
begin
  result := inherited;
  TEdit(result).TextAlign := TTextAlign.taTrailing;
end;

{TfrmMain}
procedure TfrmMain.FormShow(Sender: TObject);
begin
  OnShow := nil;
  ClientHeight := 562;
  ClientWidth  := 784;

  with sgMatchResult do
    begin
      AddObject(TStringColumn_Center.Create(sgMatchResult));
      AddObject(TStringColumn.Create(sgMatchResult));
      AddObject(TStringColumn.Create(sgMatchResult));
      AddObject(TStringColumn_Right.Create(sgMatchResult));
      AddObject(TStringColumn_Right.Create(sgMatchResult));
      Columns[0].Header := 'GroupIndex';
      Columns[1].Header := 'GroupName';
      Columns[2].Header := 'Item.Value';
      Columns[3].Header := 'Item.Index';
      Columns[4].Header := 'Item.Length';
      Columns[0].Width  :=  73;
      Columns[1].Width  :=  84;
      Columns[2].Width  := 472;
      Columns[3].Width  :=  68;
      Columns[4].Width  :=  68;
    end;

  btnMatch.Enabled := False;
  btnNextMatch.Enabled := False;
  ClearResult;
end;

procedure TfrmMain.btnEscapeClick(Sender: TObject);
begin
  edExpression.Text := TRegEx.Escape(edExpression.Text);
end;

procedure TfrmMain.btnMatchClick(Sender: TObject);
var
  Flg1: Boolean;
  Options: TRegExOptions;
begin
  pnlHeader.Enabled := False;
  try
    Options := [];
    if chbIgnoreCase.IsChecked then
      Include(Options, roIgnoreCase        );
    if chbMultiLine.IsChecked then
      Include(Options, roMultiLine         );
    if chbSingleLine.IsChecked then
      Include(Options, roSingleLine        );
    if chbExplicitCapture.IsChecked then
      Include(Options, roExplicitCapture   );
    if chbCompiled.IsChecked then
      Include(Options, roCompiled          );
    if chbIgnorePatternSpace.IsChecked then
      Include(Options, roIgnorePatternSpace);
    meInputString.Lines.LineBreak := #$0A;
    Match := RegExp.Match(meInputString.Lines.Text, edExpression.Text, Options);
    Flg1 := Match.Success;
    btnNextMatch.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    pnlHeader.Enabled := True;
  end;
end;

procedure TfrmMain.btnNextMatchClick(Sender: TObject);
var
  Flg1: Boolean;
begin
  pnlHeader.Enabled := False;
  try
    Match := Match.NextMatch;
    Flg1 := Match.Success;
    btnNextMatch.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    pnlHeader.Enabled := True;
  end;
end;

procedure TfrmMain.chb_Click(Sender: TObject);
begin
  btnNextMatch.Enabled := False;
end;

procedure TfrmMain.edExpressionChange(Sender: TObject);
begin
  btnMatch.Enabled := Length(edExpression.Text) > 0;
  btnNextMatch.Enabled := False;
end;

procedure TfrmMain.meInputStringChange(Sender: TObject);
begin
  btnNextMatch.Enabled := False;
end;

procedure TfrmMain.ClearResult;
var
  i: Integer;
begin
  sgMatchResult.RowCount := 1;
  for i:=0 to sgMatchResult.ColumnCount-1 do
    sgMatchResult.Cells[i, 0] := '';
end;

procedure TfrmMain.ShowResult(aMatch: Boolean);
var
  i: Integer;
begin
  if aMatch then
    begin
      lblSimpleText.Text := 'Match';
      if Match.Groups.Count > 0 then
        sgMatchResult.RowCount := Match.Groups.Count
      else
        sgMatchResult.RowCount := 1;
      for i:=0 to Match.Groups.Count-1 do
        begin
           sgMatchResult.Cells[0, i] := IntToStr(i);
           {$IFDEF USE_SKREGEXP}
           sgMatchResult.Cells[1, i] := Match.Groups.Item[i].GroupName;
           {$ELSE}
           sgMatchResult.Cells[1, i] := '';
           {$ENDIF}
           sgMatchResult.Cells[2, i] := Match.Groups.Item[i].Value;
           sgMatchResult.Cells[3, i] := IntToStr(Match.Groups.Item[i].Index);
           sgMatchResult.Cells[4, i] := IntToStr(Match.Groups.Item[i].Length);
        end;
      meInputString.SelStart  := Match.Groups.Item[0].Index - 1;
      meInputString.SelLength := Match.Groups.Item[0].Length;
    end
  else
    begin
      lblSimpleText.Text := 'No match';
      ClearResult;
      meInputString.SelLength := 0;
      MessageDlg('No match', TMsgDlgType.mtError, [TMsgDlgBtn.mbOK], -1);
    end;
end;


end.
