unit frmuMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, Grids, Menus, RegularExpressionsCore;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    chbExtended: TCheckBox;
    chbCaseless: TCheckBox;
    chbMultiLine: TCheckBox;
    chbSingleLine: TCheckBox;
    chbAnchored: TCheckBox;
    chbUngreedy: TCheckBox;
    edRegEx: TEdit;
    lblRegEx: TLabel;
    sbStatus: TStatusBar;
    sgMatchResult: TStringGrid;
    btnMatch: TButton;
    btnAgain: TButton;
    meSubject: TMemo;
    FontDialog1: TFontDialog;
    PopupMenu1: TPopupMenu;
    FontChange1: TMenuItem;
    btnEscape: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure edRegExChange(Sender: TObject);
    procedure btnMatchClick(Sender: TObject);
    procedure btnAgainClick(Sender: TObject);
    procedure chb_Click(Sender: TObject);
    procedure FontChange1Click(Sender: TObject);
    procedure meSubjectChange(Sender: TObject);
    procedure btnEscapeClick(Sender: TObject);
  private
    { Private 錾 }
    RegExp: TPerlRegEx;
    procedure ShowResult(aMatch: Boolean);
  public
    { Public 錾 }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.btnEscapeClick(Sender: TObject);
begin
  edRegEx.Text := TPerlRegEx.EscapeRegExChars(edRegEx.Text);
end;

procedure TfrmMain.btnMatchClick(Sender: TObject);
var
  Flg1: Boolean;
begin
  Enabled := False;
  try
    RegExp.Options := [];
    if chbExtended.Checked then
      RegExp.Options := RegExp.Options + [preExtended];
    if chbCaseless.Checked then
      RegExp.Options := RegExp.Options + [preCaseLess];
    if chbMultiLine.Checked then
      RegExp.Options := RegExp.Options + [preMultiLine];
    if chbSingleLine.Checked then
      RegExp.Options := RegExp.Options + [preSingleLine];
    if chbAnchored.Checked then
      RegExp.Options := RegExp.Options + [preAnchored];
    if chbUngreedy.Checked then
      RegExp.Options := RegExp.Options + [preUngreedy];
    RegExp.Subject := UTF8String(meSubject.Lines.Text);
    RegExp.RegEx   := UTF8String(edRegEx.Text);
    Flg1 := RegExp.Match;
    btnAgain.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    Enabled := True;
  end;
end;

procedure TfrmMain.btnAgainClick(Sender: TObject);
var
  Flg1: Boolean;
begin
  Enabled := False;
  try
    Flg1 := RegExp.MatchAgain;
    btnAgain.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    Enabled := True;
  end;
end;

procedure TfrmMain.chb_Click(Sender: TObject);
begin
  btnAgain.Enabled := False;
end;

procedure TfrmMain.edRegExChange(Sender: TObject);
begin
  btnMatch.Enabled := Length(edRegEx.Text) > 0;
  btnAgain.Enabled := False;
end;

procedure TfrmMain.FontChange1Click(Sender: TObject);
begin
  FontDialog1.Font.Assign(Self.Font);
  if not FontDialog1.Execute then
    Exit;
  Self.Font.Assign(FontDialog1.Font);
  sgMatchResult.DefaultRowHeight := edRegEx.Height;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  RegExp.Free;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OnShow := nil;
  ClientHeight := 562;
  ClientWidth  := 784;

  Caption := Caption + Format(' (ver %.1f)', [CompilerVersion]);

  RegExp := TPerlRegEx.Create;
  chbExtended.Checked   := (preExtended   in RegExp.Options);
  chbCaseless.Checked   := (preCaseLess   in RegExp.Options);
  chbMultiLine.Checked  := (preMultiLine  in RegExp.Options);
  chbSingleLine.Checked := (preSingleLine in RegExp.Options);
  chbAnchored.Checked   := (preAnchored   in RegExp.Options);
  chbUngreedy.Checked   := (preUngreedy   in RegExp.Options);

  sgMatchResult.DefaultRowHeight := edRegEx.Height;
  with sgMatchResult do
    begin
      Cells[0, 0] := 'GroupIndex';
      Cells[1, 0] := 'GroupNames';
      Cells[2, 0] := 'Groups';
      Cells[3, 0] := 'GroupOffsets';
      Cells[4, 0] := 'GroupLengths';
    end;

  btnMatch.Enabled := False;
  btnAgain.Enabled := False;
end;

procedure TfrmMain.meSubjectChange(Sender: TObject);
begin
  btnAgain.Enabled := False;
end;

procedure TfrmMain.ShowResult(aMatch: Boolean);
var
  i: Integer;
begin
  if aMatch then
    begin
      sbStatus.SimpleText := 'Match';
      if RegExp.GroupCount > 0 then
        sgMatchResult.RowCount := RegExp.GroupCount + 2
      else
        sgMatchResult.RowCount := 2;
      for i:=0 to RegExp.GroupCount do
        begin
           sgMatchResult.Cells[0, i+1] := IntToStr(i);
           sgMatchResult.Cells[1, i+1] := '';
           sgMatchResult.Cells[2, i+1] := UTF8String(RegExp.Groups[i]);
           sgMatchResult.Cells[3, i+1] := IntToStr(RegExp.GroupOffsets[i]);
           sgMatchResult.Cells[4, i+1] := IntToStr(RegExp.GroupLengths[i]);
        end;
      meSubject.SelStart  := RegExp.MatchedOffset-1;
      meSubject.SelLength := RegExp.MatchedLength;
    end
  else
    begin
      sbStatus.SimpleText := 'No match';
      sgMatchResult.RowCount := 2;
      for i:=0 to sgMatchResult.ColCount-1 do
        sgMatchResult.Cells[i, 1] := '';
      meSubject.SelLength := 0;
      MessageBox(Handle, 'No match', 'Confirm',  MB_OK + MB_ICONINFORMATION + MB_TASKMODAL);
    end;
end;


end.
