unit frmuMain;

interface

{$DEFINE USE_SKREGEXP_VER2}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, Grids, SkRegExpW, Menus;

type
  TfrmMain = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    chbExtended: TCheckBox;
    chbIgnoreCase: TCheckBox;
    chbIgnoreKana: TCheckBox;
    chbIgnoreWidth: TCheckBox;
    chbMultiLine: TCheckBox;
    chbSingleLine: TCheckBox;
    chbNamedGroupOnly: TCheckBox;
    chbDefinedCharClassLegacy: TCheckBox;
    lblExpression: TLabel;
    sbStatus: TStatusBar;
    sgMatchResult: TStringGrid;
    btnExecute: TButton;
    btnNext: TButton;
    meInputString: TMemo;
    FontDialog1: TFontDialog;
    PopupMenu1: TPopupMenu;
    FontChange1: TMenuItem;
    edExpression: TEdit;
    btnEscape: TButton;
    procedure FormShow(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure edExpressionChange(Sender: TObject);
    procedure btnExecuteClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure chb_Click(Sender: TObject);
    procedure FontChange1Click(Sender: TObject);
    procedure meInputStringChange(Sender: TObject);
    procedure btnEscapeClick(Sender: TObject);
  private
    { Private 錾 }
    RegExp: TSkRegExp;
    procedure ShowResult(aMatch: Boolean);
  public
    { Public 錾 }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.btnEscapeClick(Sender: TObject);
begin
  edExpression.Text := TSkRegExp.EscapeRegExChars(edExpression.Text);
end;

procedure TfrmMain.btnExecuteClick(Sender: TObject);
var
  Flg1: Boolean;
begin
  Enabled := False;
  try
    RegExp.Extended               := chbExtended.Checked;
    RegExp.IgnoreCase             := chbIgnoreCase.Checked;
    RegExp.IgnoreKana             := chbIgnoreKana.Checked;
    RegExp.IgnoreWidth            := chbIgnoreWidth.Checked;
    RegExp.MultiLine              := chbMultiLine.Checked;
    RegExp.SingleLine             := chbSingleLine.Checked;
    RegExp.NamedGroupOnly         := chbNamedGroupOnly.Checked;
    RegExp.DefinedCharClassLegacy := chbDefinedCharClassLegacy.Checked;
    RegExp.Expression             := edExpression.Text;
    Flg1 := RegExp.Exec(meInputString.Lines.Text);
    btnNext.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    Enabled := True;
  end;
end;

procedure TfrmMain.btnNextClick(Sender: TObject);
var
  Flg1: Boolean;
begin
  Enabled := False;
  try
    Flg1 := RegExp.ExecNext;
    btnNext.Enabled := Flg1;
    ShowResult(Flg1);
  finally
    Enabled := True;
  end;
end;

procedure TfrmMain.chb_Click(Sender: TObject);
begin
  btnNext.Enabled := False;
end;

procedure TfrmMain.edExpressionChange(Sender: TObject);
begin
  btnExecute.Enabled := Length(edExpression.Text) > 0;
  btnNext.Enabled := False;
end;

procedure TfrmMain.FontChange1Click(Sender: TObject);
begin
  FontDialog1.Font.Assign(Self.Font);
  if not FontDialog1.Execute then
    Exit;
  Self.Font.Assign(FontDialog1.Font);
  sgMatchResult.DefaultRowHeight := edExpression.Height;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  RegExp.Free;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OnShow := nil;
  ClientHeight := 562;
  ClientWidth  := 784;

  RegExp := TSkRegExp.Create;
  Caption := Caption + Format(' (ver %s)', [RegExp.Version]);
  chbExtended.Checked               := RegExp.Extended;
  chbIgnoreCase.Checked             := RegExp.IgnoreCase;
  chbIgnoreKana.Checked             := RegExp.IgnoreKana;
  chbIgnoreWidth.Checked            := RegExp.IgnoreWidth;
  chbMultiLine.Checked              := RegExp.MultiLine;
  chbSingleLine.Checked             := RegExp.SingleLine;
  chbNamedGroupOnly.Checked         := RegExp.NamedGroupOnly;
  chbDefinedCharClassLegacy.Checked := RegExp.DefinedCharClassLegacy;

  sgMatchResult.DefaultRowHeight := edExpression.Height;
  with sgMatchResult do
    begin
      Cells[0, 0] := 'GroupIndex';
      Cells[1, 0] := 'GroupName';
      Cells[2, 0] := 'Match';
      Cells[3, 0] := 'MatchPos';
      Cells[4, 0] := 'MatchLen';
    end;

  btnExecute.Enabled := False;
  btnNext.Enabled := False;
end;

procedure TfrmMain.meInputStringChange(Sender: TObject);
begin
  btnNext.Enabled := False;
end;

procedure TfrmMain.ShowResult(aMatch: Boolean);
var
  i: Integer;
begin
  if aMatch then
    begin
      sbStatus.SimpleText := 'Match';
      if RegExp.GroupCount > 0 then
        sgMatchResult.RowCount := RegExp.GroupCount + 2
      else
        sgMatchResult.RowCount := 2;
      for i:=0 to RegExp.GroupCount do
        begin
           sgMatchResult.Cells[0, i+1] := IntToStr(i);
           sgMatchResult.Cells[1, i+1] := RegExp.GroupNameFromIndex[i];
           {$IFDEF USE_SKREGEXP_VER2}
           sgMatchResult.Cells[2, i+1] := RegExp.Groups[i].Strings;
           sgMatchResult.Cells[3, i+1] := IntToStr(RegExp.Groups[i].Index);
           sgMatchResult.Cells[4, i+1] := IntToStr(RegExp.Groups[i].Length);
           {$ELSE}
           sgMatchResult.Cells[2, i+1] := RegExp.Match[i];
           sgMatchResult.Cells[3, i+1] := IntToStr(RegExp.MatchPos[i]);
           sgMatchResult.Cells[4, i+1] := IntToStr(RegExp.MatchLen[i]);
           {$ENDIF}
        end;
      {$IFDEF USE_SKREGEXP_VER2}
      meInputString.SelStart  := RegExp.Groups[0].Index - 1;
      meInputString.SelLength := RegExp.Groups[0].Length;
      {$ELSE}
      meInputString.SelStart  := RegExp.MatchPos[0] - 1;
      meInputString.SelLength := RegExp.MatchLen[0];
      {$ENDIF}
    end
  else
    begin
      sbStatus.SimpleText := 'No match';
      sgMatchResult.RowCount := 2;
      for i:=0 to sgMatchResult.ColCount-1 do
        sgMatchResult.Cells[i, 1] := '';
      meInputString.SelLength := 0;
      MessageBox(Handle, 'No match', 'Confirm',  MB_OK + MB_ICONINFORMATION + MB_TASKMODAL);
    end;
end;


end.
